import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_markdown/flutter_markdown.dart';
import 'package:provider/provider.dart';
import 'package:shimmer/shimmer.dart';
import 'package:url_launcher/url_launcher.dart';

import 'states/auth_state.dart';

final String tag = 'tag LogInWithEmail';

class EmailLoginScreen extends StatefulWidget {
  final AppBar appBar;
  final String privacyUrl, termsUrl;
  final Color mainColor, shimmerColor1, shimmerColor2;

  const EmailLoginScreen(
      {Key key,
      this.appBar,
      @required this.privacyUrl,
      @required this.termsUrl,
      @required this.mainColor, //not white
      this.shimmerColor1,
      this.shimmerColor2})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => new _EmailLoginScreenState();
}

enum _Mode { LOGIN, SIGN_UP }

class _EmailLoginScreenState extends State<EmailLoginScreen> {
  final _scaffoldKey = GlobalKey<ScaffoldState>();
  final _formKey = GlobalKey<FormState>();

  AuthState _authState;

  bool _isEmailRegistered;
  String _email;
  String _password;
  String _name;

  String _errorMessage;

  bool _loading;

  @override
  void initState() {
    _loading = false;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    _authState = Provider.of<AuthState>(context);

    _accessWithEmail(_Mode mode) async {
      setState(() {
        _loading = true;
        _errorMessage = "";
      });

      try {
        if (mode == _Mode.LOGIN) {
          await _authState.signInWithEmail(_email, _password);
        } else {
          await _authState.signUpWithEmail(_email, _password, _name);
        }

        setState(() {
          _loading = false;
        });

        Navigator.pop(context);
      } catch (e) {
        /// Errors:
        ///   • `ERROR_WEAK_PASSWORD` - If the password is not strong enough.
        ///   • `ERROR_INVALID_EMAIL` - If the email address is malformed.
        ///   • `ERROR_EMAIL_ALREADY_IN_USE` - If the email is already in use by a different account.
        ///
        ///
        ///   • `ERROR_INVALID_EMAIL` - If the [email] address is malformed.
        ///   • `ERROR_WRONG_PASSWORD` - If the [password] is wrong.
        ///   • `ERROR_USER_NOT_FOUND` - If there is no user corresponding to the given [email] address, or if the user has been deleted.
        ///   • `ERROR_USER_DISABLED` - If the user has been disabled (for example, in the Firebase console)
        ///   • `ERROR_TOO_MANY_REQUESTS` - If there was too many attempts to sign in as this user.
        ///   • `ERROR_OPERATION_NOT_ALLOWED` - Indicates that Email & Password accounts are not enabled.

        String errorCode = e.code;
        String message = e.message;

        switch (errorCode) {
          case "ERROR_WEAK_PASSWORD":
            message = 'La contraseña no es segura';
            break;
          case "ERROR_INVALID_EMAIL":
            message = 'Email no válido';
            break;
          case "ERROR_EMAIL_ALREADY_IN_USE":
            message = 'Ese email ya está en uso';
            break;
          case "ERROR_WRONG_PASSWORD":
            message = 'Contraseña errónea';
            break;
          case "ERROR_USER_NOT_FOUND":
            message = 'Usuario no encontrado';
            break;
          case "ERROR_USER_DISABLED":
            message = 'Usuario inhabilitado';
            break;
          case "ERROR_TOO_MANY_REQUESTS":
            message = 'Has hecho demasiadas peticiones, intentalo más tarde';
            break;
          case "ERROR_OPERATION_NOT_ALLOWED":
            message = 'Operación no permitida';
            break;
        }

        setState(() {
          _loading = false;
          _errorMessage = message;
        });
      }
    }

    Widget _showErrorMessage() {
      if (_errorMessage != null && _errorMessage.length > 0) {
        return Center(
          child: Text(
            _errorMessage,
            style: TextStyle(
                fontSize: 13.0,
                color: Colors.red,
                height: 1.0,
                fontWeight: FontWeight.w300),
          ),
        );
      } else {
        return Container(
          height: 0.0,
        );
      }
    }

    _showResetPasswordSnackBar(String mail) {
      final snackBar = SnackBar(
        content: Text('Te hemos enviado un emai a $mail'),
        duration: Duration(seconds: 30),
        action: SnackBarAction(
          label: 'Ok',
          onPressed: () {
            // Some code to undo the change.
            _scaffoldKey.currentState.hideCurrentSnackBar();
          },
        ),
      );
      _scaffoldKey.currentState.showSnackBar(snackBar);
    }

    _showDialogForResetPassword(String mail) {
      showDialog(
          context: context,
          builder: (BuildContext ctx) {
            return AlertDialog(
              title: Text("Cambiar contraseña"),
              content: Text(
                  "Te enviaremos un email a $mail con instrucciones sobre como cambiar la contraseña"),
              actions: <Widget>[
                FlatButton(
                  child: Text("Cancelar"),
                  onPressed: () => {Navigator.of(ctx).pop()},
                ),
                FlatButton(
                  child: Text("Aceptar"),
                  onPressed: () => {
                    setState(() => {_loading = true}),
                    _authState.resetPassword(mail).then((_) => {
                          _showResetPasswordSnackBar(mail),
                          setState(() => {_loading = false}),
                        }),
                    Navigator.of(context).pop()
                  },
                )
              ],
            );
          });
    }

    _getEmailInput() {
      return Container(
          padding: EdgeInsets.all(16.0),
          child: new Form(
              key: _formKey,
              child: ListView(
                children: <Widget>[
                  Padding(
                    padding: const EdgeInsets.fromLTRB(0, 16, 0, 32),
                    child: new TextFormField(
                      maxLines: 1,
                      keyboardType: TextInputType.emailAddress,
                      autofocus: false,
                      decoration: new InputDecoration(
                          hintText: 'Correo electrónico',
                          icon: new Icon(
                            Icons.mail,
                            color: Colors.grey,
                          )),
                      validator: (value) {
                        if (value.isEmpty) {
                          return 'El email no puede estar vacío';
                        } else if (!isValidEmail(value)) {
                          return 'Email no válido';
                        }
                        return null;
                      },
                      onSaved: (value) => _email = value,
                    ),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: <Widget>[
                      RaisedButton(
                        onPressed: () async {
                          if (_formKey.currentState.validate()) {
                            _formKey.currentState.save();
                            setState(() {
                              _loading = true;
                            });
                            _isEmailRegistered =
                                await _authState.isEmailRegistered(_email);
                            print("$_email $_isEmailRegistered");
                            setState(() {
                              _loading = false;
                            });
                          }
                        },
                        color: widget.mainColor,
                        child: Text(
                          "SIGUIENTE",
                          style: TextStyle(color: Colors.white),
                        ),
                      )
                    ],
                  )
                ],
              )));
    }

    _getPasswordInput() {
      return Container(
          padding: EdgeInsets.all(16.0),
          child: new Form(
              key: _formKey,
              child: ListView(
                children: <Widget>[
                  Container(
                    width: double.infinity,
                    height: 110,
                    child: Markdown(
                      data:
                          "## ¡Hola de nuevo!\nYa has usado __${_email}__ para iniciar sesión. Introduce la contraseña de esta cuenta.",
                      styleSheet:
                          MarkdownStyleSheet.fromTheme(Theme.of(context))
                              .copyWith(
                                  p: Theme.of(context)
                                      .textTheme
                                      .body1
                                      .copyWith(fontSize: 16)),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.fromLTRB(0, 16, 0, 32),
                    child: new TextFormField(
                      maxLines: 1,
                      obscureText: true,
                      autofocus: false,
                      decoration: new InputDecoration(
                          hintText: 'Contraseña',
                          icon: new Icon(
                            Icons.lock,
                            color: Colors.grey,
                          )),
                      validator: (value) {
                        if (value.isEmpty) {
                          return 'Introduce una contraseña';
                        }

                        //Esta comprobación no se hace en sign up
                        /*if(value.length<6){
                          return "La contraseña es demasiado corta";
                        }*/

                        return null;
                      },
                      onSaved: (value) => _password = value,
                    ),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: <Widget>[
                      Expanded(
                        child: Container(
                          padding: EdgeInsets.only(right: 16),
                          child: FlatButton(
                            child: Text(
                              "¿Olvidaste la contraseña?",
                              style: TextStyle(color: widget.mainColor),
                            ),
                            onPressed: () {
                              _showDialogForResetPassword(_email);
                            },
                          ),
                        ),
                      ),
                      RaisedButton(
                        onPressed: () async {
                          if (_formKey.currentState.validate()) {
                            _formKey.currentState.save();
                            print("$_email $_password");
                            await _accessWithEmail(_Mode.LOGIN);
                          }
                        },
                        color: widget.mainColor,
                        child: Text(
                          "SIGUIENTE",
                          style: TextStyle(color: Colors.white),
                        ),
                      )
                    ],
                  ),
                  Padding(
                    padding: EdgeInsets.all(16),
                    child: _showErrorMessage(),
                  ),
                ],
              )));
    }

    _getSignUpInput() {
      String _markDownPrivacy =
          "Al continuar aceptas la [política de privacidad](${widget.privacyUrl}) "
          "y las [condiciones de servicio](${widget.termsUrl}).";

      return Container(
          padding: EdgeInsets.all(16.0),
          child: new Form(
              key: _formKey,
              child: ListView(
                children: <Widget>[
                  Padding(
                    padding: const EdgeInsets.fromLTRB(0, 16, 0, 16),
                    child: new TextFormField(
                      maxLines: 1,
                      keyboardType: TextInputType.text,
                      textCapitalization: TextCapitalization.words,
                      autofocus: false,
                      decoration: new InputDecoration(
                          hintText: 'Nombre',
                          icon: new Icon(
                            Icons.person,
                            color: Colors.grey,
                          )),
                      validator: (value) {
                        var name = value.trim();
                        if (name.isEmpty) {
                          return 'El nombre no puede estar vacío';
                        }
                        return null;
                      },
                      onSaved: (value) => _name = value.trim(),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.fromLTRB(0, 16, 0, 16),
                    child: new TextFormField(
                      maxLines: 1,
                      obscureText: true,
                      autofocus: false,
                      decoration: new InputDecoration(
                          hintText: 'Contraseña',
                          icon: new Icon(
                            Icons.lock,
                            color: Colors.grey,
                          )),
                      validator: (value) {
                        if (value.isEmpty) {
                          return 'Introduce una contraseña';
                        } else if (value.length < 6) {
                          return "La contraseña es demasiado corta";
                        }
                        return null;
                      },
                      onSaved: (value) => _password = value,
                    ),
                  ),
                  Padding(
                    padding: EdgeInsets.all(8),
                    child: _showErrorMessage(),
                  ),
                  Container(
                    height: 60,
                    child: Markdown(
                      data: _markDownPrivacy,
                      onTapLink: (url) async {
                        if (await canLaunch(url)) {
                          await launch(url);
                        } else {
                          throw 'Could not launch $url';
                        }
                      },
                    ),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: <Widget>[
                      RaisedButton(
                        onPressed: () async {
                          if (_formKey.currentState.validate()) {
                            _formKey.currentState.save();
                            print("$_email $_password");
                            await _accessWithEmail(_Mode.SIGN_UP);
                          }
                        },
                        color: widget.mainColor,
                        child: Text(
                          "SIGUIENTE",
                          style: TextStyle(color: Colors.white),
                        ),
                      )
                    ],
                  ),
                ],
              )));
    }

    var _mainWidget = Container();
    if (_email == null && _isEmailRegistered == null) {
      _mainWidget = _getEmailInput();
    } else if (_isEmailRegistered != null && _isEmailRegistered) {
      _mainWidget = _getPasswordInput();
    } else if (_isEmailRegistered != null && !_isEmailRegistered) {
      _mainWidget = _getSignUpInput();
    }

    return Scaffold(
        key: _scaffoldKey,
        appBar: widget.appBar,
        body: Stack(
          children: <Widget>[
            _mainWidget,
            _loading
                ? Shimmer.fromColors(
                    child: Container(
                      height: 6,
                      color: Colors.white,
                    ),
                    baseColor: widget.shimmerColor1 != null
                        ? widget.shimmerColor1
                        : Theme.of(context).primaryColor,
                    highlightColor: widget.shimmerColor2 != null
                        ? widget.shimmerColor2
                        : Colors.white,
                    period: Duration(milliseconds: 750),
                  )
                : Container(),
          ],
        ));
  }
}

bool isValidEmail(String email) {
  return RegExp(r"^[a-zA-Z0-9.]+@[a-zA-Z0-9]+\.[a-zA-Z]+").hasMatch(email);
}
